﻿using UnityEngine;
using System.Collections;


public class LevelManager : MonoBehaviour
{
    public GameObject PrefabGround;
    public GameObject PrefabObstacle;
    public GameObject PrefabSeed1;
    public GameObject PrefabSeed2;
    public GameObject PrefabSeed3;
    public GameObject PrefabPlayer;
    public GameObject PrefabBridge;

    public const float BridgeY1 = 0.3f;
    public const float BridgeY2 = 1.1f;

    GameObject _levelRoot = null;

    int _offsetX;
    int _offsetZ;

    int _widthX;
    public static int WidthX { get { return _singleton._widthX; } }

    int _widthZ;
    public static int WidthZ { get { return _singleton._widthZ; } }

    bool[,] _levelObstacle;
    bool[,] _levelSeed;
    int[,] _levelBridge;

    public int _nbSeeds;
    public static int NbSeeds { get { return _singleton._nbSeeds; } }

    static LevelManager _singleton = null;


    // Use this for initialization
    void Start ()
    {
        if (_singleton != null)
        {
            Debug.LogError("Another LevelManager is already instanciated on this scene");
        }
        else
        {
            _singleton = this;
        }

        GenerateLevel(GlobalData.CurrentLevel);
	}
	
	// Update is called once per frame
	void Update ()
    {	
	}

    void GenerateLevel(int numLevel)
    {
        Random.seed = numLevel + 1;

        int minSize = 7;
        int addSizeX = Random.Range(numLevel, numLevel * 4) / 5;
        addSizeX = (numLevel % 2) == 1 ? Mathf.Max(addSizeX, numLevel - addSizeX) : Mathf.Min(addSizeX, numLevel - addSizeX);
        int addSizeZ = numLevel - addSizeX;
        _widthX = minSize + addSizeX - (addSizeX % 2);
        _widthZ = minSize + addSizeZ - (addSizeZ % 2);

        Debug.LogFormat("GenerateLevel({0}) : widthX={1}, widthZ={2}\n", numLevel, _widthX, _widthZ);

        _levelObstacle = new bool[_widthX,_widthZ];
        _levelSeed = new bool[_widthX, _widthZ];
        _levelBridge = new int[_widthX, _widthZ];

        // Initialize with default value;
        for (int x = 1; x < _widthX; x += 2)
        {
            for (int z = 1; z < _widthZ; z += 2)
            {
                _levelObstacle[x, z] = false;
                _levelSeed[x, z] = false;
                _levelBridge[x, z] = 0;
            }
        }

        _levelRoot = new GameObject("LevelRoot" + numLevel);
        _levelRoot.transform.parent = transform;

        GameObject ground = Instantiate<GameObject>(PrefabGround);
        ground.transform.parent = _levelRoot.transform;
        ground.transform.localScale = new Vector3(_widthX, 1, _widthZ);

        _offsetX = -_widthX / 2;
        _offsetZ = -_widthZ / 2;

        // Reset the total number of seeds for the level.
        _nbSeeds = 0;

        // [1] Always put obstacle at odd coordinates.
        for (int x = 0; x < _widthX; x++)
        {
            for (int z = 0; z < _widthZ; z++)
            {
                if (x % 2 == 1 && z % 2 == 1)
                {
                    _levelObstacle[x, z] = true;
                }
            }
        }

        int startX = (LevelManager.WidthX / 4) * 2;
        int startZ = (LevelManager.WidthZ / 4) * 2;

        // [2] Put random obstacles except on borders.
        for (int x = 2; x < _widthX - 2; x++)
        {
            for (int z = 2; z < _widthZ - 2; z++)
            {
                if (x % 2  != z % 2)
                {
                    // Prevent obstacles to be put on front of the player at start.
                    if (x != startX || z < startZ)
                    {
                        // Prevent mutiples obstacles to create a dead-end.
                        if ((x % 2 == 1 && CheckObstacleAlgo1(x - 1, z) && CheckObstacleAlgo1(x + 1, z)) ||
                            (x % 2 == 0 && CheckObstacleAlgo1(x, z - 1) && CheckObstacleAlgo1(x, z + 1)))
                        {
                            if (Random.value <= 0.50f)
                            {
                                _levelObstacle[x, z] = true;
                            }
                        }
                    }
                }
            }
        }

        //if (numLevel == 1)
        //{
        //    Debug.Log("Hardcoded bridge for test !!!!!!\n");
        //    InstanciateBridge(2, 1, 0);
        //    InstanciateBridge(1, 2, 1);
        //    InstanciateBridge(4, 3, 0);
        //}

        // Bridges starts at level 4
        if (numLevel >= 4)
        {
            PlaceBridge(2, _widthX - 2, 1, _widthZ - 3, 0, 2, 0);
            PlaceBridge(1, _widthX - 3, 2, _widthZ - 2, 2, 0, 1);
            PlaceBridge(2, _widthX - 2, 3, _widthZ - 1, 0, -2, 2);
            PlaceBridge(3, _widthX - 1, 2, _widthZ - 2, -2, 0, 3);
        }

        for (int x = 0; x < _widthX; x++)
        {
            for (int z = 0; z < _widthZ; z++)
            {
                if (_levelObstacle[x, z])
                {
                    InstanciateObstacle(x, z);
                }
                else
                {
                    // If there is no obstacle, put seed
                    InstanciateSeed(x, z);
                }
            }
        }

        Instantiate<GameObject>(PrefabPlayer);
    }

    void PlaceBridge(int startX, int endX, int startZ, int endZ, int checkX, int checkZ, int dir)
    {
        for (int x = startX; x < endX; x += 2)
        {
            for (int z = startZ; z < endZ; z += 2)
            {
                if (_levelObstacle[x, z] && _levelObstacle[x + checkX, z + checkZ] == false)
                {
                    if (Random.value <= 0.30f)
                    {
                        _levelObstacle[x, z] = false;
                        InstanciateBridge(x, z, dir);
                    }
                }
            }
        }
        //for (int x = 2; x < _widthX - 2; x += 2)
        //{
        //    for (int z = 1; z < _widthZ - 3; z += 2)
        //    {
        //        if (_levelObstacle[x, z] && _levelObstacle[x, z + 2] == false)
        //        {
        //            //if (Random.value <= 0.50f)
        //            {
        //                _levelObstacle[x, z] = false;
        //                InstanciateBridge(x, z, 0);
        //            }
        //        }
        //    }
        //}
    }

    bool CheckObstacleAlgo1(int x, int z)
    {
        int nb = 0;

        if (_levelObstacle[x -1, z])
            nb++;
        if (_levelObstacle[x + 1, z])
            nb++;
        if (_levelObstacle[x, z - 1])
            nb++;
        if (_levelObstacle[x, z + 1])
            nb++;

        return nb < 2;
    }

    void InstanciateObstacle(int x, int z)
    {
        GameObject obstacle = Instantiate<GameObject>(PrefabObstacle);
        obstacle.name += string.Format("[{0},{1}]", x, z);
        obstacle.transform.parent = _levelRoot.transform;
        obstacle.transform.position = WorldPosition(x, z);
    }

    void InstanciateBridge(int x, int z, int dir)
    {
        _levelBridge[x, z] = dir + 1;
        GameObject bridge = Instantiate<GameObject>(PrefabBridge);
        bridge.name += string.Format("[{0},{1}]", x, z);
        bridge.transform.parent = _levelRoot.transform;
        bridge.transform.position = WorldPosition(x, z);
        bridge.transform.rotation = Quaternion.Euler(0, dir * 90, 0);
    }

    void InstanciateSeed(int x, int z)
    {
        _nbSeeds++;
        _levelSeed[x, z] = true;

        GameObject seed;
        if (GlobalData.CurrentLevel < 8)
        {
            seed = Instantiate<GameObject>(PrefabSeed1);
        }
        else if (GlobalData.CurrentLevel < 16)
        {
            seed = Instantiate<GameObject>(PrefabSeed2);
        }
        else
        {
            seed = Instantiate<GameObject>(PrefabSeed3);
        }

        seed.name += string.Format("[{0},{1}]", x, z);
        seed.transform.parent = _levelRoot.transform;
        seed.transform.position = WorldPosition(x, z);
        if (_levelBridge[x, z] != 0)
        {
            seed.transform.position += new Vector3(0, BridgeY1, 0);
        }
    }

    public static Vector3 WorldPosition(int x, int z)
    {
        return new Vector3(x + _singleton._offsetX, 0, z + _singleton._offsetZ);
    }

    public static bool LevelObstacle(int x, int z)
    {
        if (x < 0 || x >= _singleton._widthX || z < 0 || z >= _singleton._widthZ)
            return true;
        else
            return _singleton._levelObstacle[x, z];
    }

    public static int LevelBridge(int x, int z)
    {
        if (x < 0 || x >= _singleton._widthX || z < 0 || z >= _singleton._widthZ)
            return 0;
        else
            return _singleton._levelBridge[x, z];
    }
}
