﻿using UnityEngine;
using System.Collections;

public interface ITargetHost
{
    Vector3 GetTarget(int index);
    float GetSpeed();
    float GetRotationSpeed();
}

public class TargetFollower : MonoBehaviour
{
    ITargetHost _targetHost;

    public AudioClip SoundExplode;

    [SerializeField]
    Vector3 _targetPosition;
    [SerializeField]
    Quaternion _targetRotation;

    [SerializeField]
    public int TargetIndex;

    float _slowUntilTime;

    [SerializeField]
    public float TotalDistance;

    float _offsetY;
    float _amplitudeY = 0.1f;
    float _cycleY = 1.5f;

    bool _isExploding = false;

    // Use this for initialization
    void Start ()
    {
    }

    // Update is called once per frame
    void Update ()
    {
        if (_isExploding)
        {
            return;
        }

        float speed = _targetHost.GetSpeed();
        if (Time.time < _slowUntilTime)
        {
            speed /= 2;
        }

        Vector3 position = GetPosition();

        float newOffsetY = OffsetY();
        float maxMove = Time.deltaTime * (speed - Mathf.Abs(_offsetY - newOffsetY));
        _offsetY = newOffsetY;

        // Move to target position
        if (position != _targetPosition)
        {
            float move = (position - _targetPosition).magnitude;
            move = Mathf.Min(move, maxMove);
            maxMove -= move;
            TotalDistance += move;
            position = Vector3.MoveTowards(position, _targetPosition, move);
        }

        // Turn to target direction
        if (transform.rotation != _targetRotation)
        {
            transform.rotation = Quaternion.Slerp(transform.rotation, _targetRotation, Time.deltaTime * _targetHost.GetRotationSpeed());
        }

        // Get a new target if the actual one is reached.
        if (position == _targetPosition)
        {
            TargetIndex++;
            FollowTarget(position);
        }

        // Move to new target position if there is some movement left to avoid speed change during the frame of a target change.
        if (maxMove > 0)
        {
            TotalDistance += maxMove;
            position = Vector3.MoveTowards(position, _targetPosition, maxMove);
        }
        position.y += _offsetY;
        transform.position = position;
    }

    public void FollowTarget(Vector3 actualPosition)
    {
        _targetPosition = _targetHost.GetTarget(TargetIndex);
        Vector3 direction = _targetPosition - actualPosition;
        if (direction == Vector3.zero)
        {
            Debug.LogFormat("No more taget to follow : Explode {0}\n", name);
            _isExploding = true;
            StartCoroutine(Explode());
        }
        else
        {
            // Don't turn up/down
            direction.y = 0;
            _targetRotation = Quaternion.LookRotation(direction);
        }
    }

    IEnumerator Explode()
    {
        GetComponentInParent<AudioSource>().PlayOneShot(SoundExplode);

        Vector3 rotation = new Vector3(Random.Range(-90,90), Random.Range(-90, 90), Random.Range(-90, 90));

        // Fly an turn around randomly
        float flySpeed = 15;
        float duration = Time.time + 3;
        while (Time.time < duration)
        {
            transform.Rotate(rotation * Time.deltaTime, Space.World);
            transform.Translate(transform.up * Time.deltaTime * flySpeed, Space.World);
            yield return 0;
        }

        Destroy(gameObject);
    }

    public Vector3 GetPosition()
    {
        return transform.position - new Vector3(0, _offsetY, 0);
    }

    float OffsetY()
    {
        float phaseY = (TotalDistance / _cycleY) * Mathf.PI;
        return _amplitudeY + _amplitudeY * Mathf.Sin(phaseY);
    }


    public void SetTargetHoster(ITargetHost targetHost, int targetIndex, float slowDuration, float totalDistance)
    {
        _targetHost = targetHost;
        TargetIndex = targetIndex;

        _slowUntilTime = Time.time + slowDuration;

        TotalDistance = totalDistance;

        _offsetY = OffsetY();

        _targetPosition = _targetHost.GetTarget(TargetIndex);
        _targetRotation = Quaternion.LookRotation(_targetPosition - transform.position);
    }
}
